
AWK                                                                    AWK


NAME

        awk - pattern scanning and processing language


SYNOPSIS

        awk [-Fstr] prog file ...


DESCRIPTION

Awk scans each input file for lines that match any of a set of patterns 
specified in prog.  With each pattern in prog there can be an associated 
action that will be performed when a line of a file matches the pattern.  
The set of patterns are read from the file prog or standard input ('-').

Files are read in order, the file name '-' means standard input.  Each line 
is matched against the pattern portion of every pattern-action statement; 
the associated action is performed for each matched pattern. 

An input line is made up of fields separated by the field separator FS.  
The fields are denoted by $1, $2 ...; $0 denotes the entire line. 

A pattern-action statement has the form:

        pattern { action }

A missing { action } means print the line, a missing pattern always 
matches.

An action is a sequence of statements.  A statement can be one of the 
following:

        break
        continue
        expression
        { statement-list }
        print expression-list
        printf format, expression-list
        while ( expression ) statement
        do statement while ( expression )
        for ( variable in array ) statement
        if ( expression ) statement [ else statement ]
        for ( expression; expression; expression) statement
        next                    #skip remaining patterns on this line
        exit [ expression ]     #skip remaining input [ exit status ]

Statements and Pattern-Actions are separated by semicolons or newlines.  
A statement may be continued on the next line by putting a \ at the end 
of the line.

The for ( i in a ) statement assigns to i the indexes of a for all elements 
in a.  The for (;;) statement is as in C as are break and continue.

The next statements stops processing the pattern action statements and reads 
in the next record.  exit will cause the END actions to be performed or if 
encountered in an END action will cause termination of the program.  The 
optional expression is returned as the exit status unless overridden by a 
further exit statement in an END action.

Expressions take on string or numeric values depending on the operators.
There is only one string operator, concatenation, indicated by adjacent 
epxressions.  The following are the operators in order of increasing 
precedence:

Operation           Operator      Example     Meaning

assignment          = *= /= %=    x += 2      two is added to x
                    += -= ^=
conditional         ?:            x?y:z       if x then y else z
logical OR          ||            x||y        1 if x else if y else 0
logical AND         &&            x&&y        1 if x if y else 0
array membership    in            x in y      1 if y[x] exists else 0
matching            ~ !~          $1~/x/      1 if $1 contains an x else 0
relational          == != >       x==y        1 if x equals y else 0
                    <= >= <
concatenation                     "x" "y"     a new string "xy"
add, subtract       +  -          x+y         sum of x and y
mul, div, mod       * / %         x*y         product of x and y
unary plus minus    + -           -x          negative of x
logical not         !             !x          1 if x is 0 or null else 0
exponentiation      ^             x^y         x to the yth power
inc, dec            ++ --         ++x, ++x    add 1 to x
field               $             $x          the xth field
grouping            ()            ($x)++      increment the xth field

Variables may be scalars, array elements (denoted x[i]) or fields (denoted 
$expression).  Variable names begin with a letter or underscore and may 
contain any number of letters, digits, or underscores.  Array subscripts may 
be any string;  this allows for a form of associative memory. 

Variables are initialized to both zero and the null string.  Fields and the 
command line arguments will be both string and numeric if they can be 
completely represented as numbers.  The range for numbers is 1E-306..1E306.

Comparison will be numeric if both operands are numeric otherwise a string 
comparison will be made.  Operands will be coerced to strings if necessary.  
Uninitialized variables will compare as numeric if the other operand is 
numeric or uninitialized.  Eg. 2 > "10" and 2 < 10.

There are a number of built in variables they are:

Variable        Meaning                                         Default

ARGC            number of command line arguments                   -
ARGV            array of command line arguments                    -
FILENAME        name of current input file                         -
FNR             record number in current file                      -
FS              controls the input field separator                " "
NF              number of fields in current record                 -
NR              number of records read so far                      -
OFMT            output format for records                        "%.6g"
OFS             output field separator                            " "
ORS             output record separator                           "\n"
RLENGTH         length of string matched by match function         -
RS              controls input record separator                   "\n"
RSTART          start of string match by match function            -
SUBSEP          subscript separator                              "\034"


There are a number of built in functions:

Function            Value returned

atan2(x, y)         arctangent of x/y           in the range -pi to pi
cos(x)              cosine of x                 x in radians
exp(x)              exponentiation of x         (e ^ x)
gsub(r, s)          number of substitutions     substitute s for all r in $0
gsub(r, s, t)       number of substitutions     substitute s for all r in t     
index(s, t)         position of t in s or 0
length(s)           number of characters in s
log(x)              natural log of x
match(s, r)         position of r in s or 0     sets RSTART and RLENGTH
rand()              random number               0 <= rand < 1
sin(x)              sine of x                   x in radians
split(s, a)         number of fields            split s into a on FS 
split(s, a, fs)     number of fields            split s into a on fs
sprintf(f, e, ...)  formatted string
sqrt(x)             square root of x
sub(r, s)           number of substitutions     substitute s for one r in $0
sub(r, s, t)        number of substitutions     substitute s for one r in t
substr(s, p)        substring of s from p to end
substr(s, p, n)     substring of s from p of length n
system(s)           exit status                 execute command s

The numeric procedure srand(x) set a new seed for the random number 
generator.  srand() set the seed from the time.

The regular expression arguments of sub, gsub, and match may be either 
regular expressions delimited by slashes or any expression.  The expression 
is coerced to a string and the resulting string is converted into a regular 
expression.

The print and printf statements come in several forms:

Form                            Meaning

print                           print $0 on standard output
print expression, ...           prints expressions separated by OFS
print(expression, ...)  
printf format, expression, ...
printf(format, expression, ...)
print >"file"                   print $0 on file "file"
print >>"file"                  append $0 to file "file"
printf(format, ...) >"file"
printf(format, ...) >>"file"

close("file")                   close the file "file"

The print statement prints its arguments on the standard output, or the 
specified file, separated by the current output field separator, and 
terminated by the output record separator.  The printf statement formats 
its expression-list according to the format.  The file is only opened once
unless it is closed between executions of the print statement.  A file must
be closed if it is to be used for input.  The "file" argument may any 
expression.

There is one function that is used for input.  It has several forms

Form                            Meaning

getline                         read the next record into $0
getline s                       read the next record into s
getline <"file"                 read a record from file "file" into $0
getline s <"file"               read a record from file "file" into s

getline returns -1 if there is an error (such as non existent file), 0 on 
end of file and 1 otherwise.

Patterns come in several forms:

Form            Example         Meaning

BEGIN           BEGIN           perform action before input is read
END             END             perform action after all input is read
expression      $1 == 3         first field is the number 3
string match    /stop/          line contains the string "stop"
compound        /x/ && NF > 2   more that two fields and contain "x"
range           NR==10,NR==20   records ten through twenty inclusive


A regular expression by itself is matched against the input record. That 
is /abc/ is equivalent to $0 ~ /abc/.

If both the program and the data are coming from standard input then a 
period on a line by itself followed by a blank line may be used to separate
the program from the data.

The field separator is a string that is interpreted as a regular expression.
A single space is changed to /[ \t]+/ and any leading spaces or tabs are
removed.  A BEGIN action may be used to set the separator or it may be set
by usining the -F command line option.

        BEGIN { FS = "," }      sets FS to a single comma
        "-F[ ]"                 sets FS to a single space

If the record separator RS is set to the null string then multi line 
records may be read in.  In this case the record separator is an empty 
line.

REGULAR EXPRESSIONS

A \ followed by a single character matches that character.

The ^ ($) matches the beginning (end) of the string.

A . matches any character.

A single character with no special meaning matches that character.

A string enclosed in brackets [] matches any single character in that 
string.  Ranges of ASCII character codes may be abbreviated as 'a-z0-9'.  
A left bracket ] may occur only as the first character of the string.  A 
literal - must be placed where it can't be mistaken as a range indicator. 
If the first character is the caret ^ then any character not in the 
string will match.

A regular expression followed by * (+, ?) matches a sequence of 0
or more (1 or more, 0 or 1) matches of the regular expression.

Two regular expressions concatenated match a match of the first 
followed by a match of the second.

Two regular expressions separated by | match either a match for the
first or a match for the second.

A regular expression enclosed in parentheses matches a match for the
regular expression.

The order of precedence of operators at the same parenthesis level is 
[] then *+? then concatenation then |.


PRINTF FORMAT

Any character except % and \ is printed as that character.

A \ followed by up to three octal digits is the ASCII character
represented by that number.

A \ followed by n, t, r, b, f, v, or p is newline, tab, return, backspace, 
form feed, vertical tab, or escape. 

%[-][number][.number][l][c|d|E|e|F|f|G|g|o|s|X|x|%] prints an expression:

The optional leading - means left justified in the field
The optional first number is the field width
The optional . and second number is the precision
The optional l denotes a long expression
The final character denotes the form of the expression

        c character
        d decimal
        e exponential floating point
        f fixed, or exponential floating point
        g decimal, fixed, or exponential floating point
        o octal
        s string
        x hexadecimal 

An upper case E, F, or G denotes use of upper case E in exponential format.
An upper case X denotest hexadecimal in upper case.
Two percent characters (%%) will print as one.

A format will match /[^%]*(%-?([0-9]+)?(.[0-9]+)?l?[cdEeFfGgosXx%][^%]*)*/

EXAMPLES

Print lines longer than 72 characters:

        length($0) > 72

Print first two fields in opposite order:

        { print $2, $1 }

Add up first column, print sum and average:

                { s = s + $1 }
        END     { print "sum is", s, "average is", s/NR }

Print fields in reverse order:

        { for (i = NF; i > 0; --i ) print $i }

Print all lines between start/stop pairs:

        /start/,/stop/

Print all lines whose first field is different from previous one:

        $1 != prev { print; prev = $1 }

Convert date from MM/DD/YY to metric (YYMMDD):

        { n = split(date, a, "/"); date = a[3] a[1] a[2] }

Copy a C program and insert include files:

        $1 == "#include" && $2 ~ /^"/ {
                gsub(/"/, "", $2);
                while (getline <$2 > 0) print
                next
        }
        { print }

AUTHOR

Rob Duff,  Vancouver, B.C.

SEE ALSO

M. E. Lesk and E. Schmidt,
        LEX - Lexical Analyser Generator

A. V Aho, B. W Kernighan, P. J. Weinberger,
        Awk - a pattern scanning and processing language

A. V Aho, B. W Kernighan, P. J. Weinberger,
        The AWK Programming Language 
        Addison-Wesley 1988   ISBN 0-201-07981-X


NOTES

There are no explicit conversions between numbers and strings.  To force an 
expression to b treated as a number add 0 to it; to force it to be a string 
concatenate "" to it.  Array indices are strings and may have the same 
numerical value but will index different values (eg "01" vs "1").  
Numerical indices will have the same value (eg 0001 vs 1.000). 

